'use strict';

var _Scope = require('./Scope');

var _Scope2 = _interopRequireDefault(_Scope);

var _debug = require('../debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

module.exports = class Executor {

  constructor(options) {
    const operations = options.operations,
          scope = options.scope,
          fetcher = options.fetcher;

    this.scope = scope || new _Scope2.default({
      parent: null,
      value: {}
    });
    this.fetch = fetcher;
    this.operations = operations;
  }

  run() {
    return this.exec(this.operations, this.scope);
  }

  exec(operation, scope) {
    var _this = this;

    return _asyncToGenerator(function* () {
      (0, _debug2.default)(operation, scope);
      switch (operation.op) {
        case 'everything':
          // Make a fake no-op filter to execute a naked everything
          return _this.execFilter({
            op: 'filter',
            source: operation,
            filter: null
          });
        case 'filter':
          return _this.execFilter(operation, scope);
        case 'and':
          return _this.execAnd(operation, scope);
        case 'eq':
          return _this.execEq(operation, scope);
        case 'accessor':
          return _this.execAccessor(operation, scope);
        case 'literal':
          return _this.execLiteral(operation, scope);
        default:
          throw `(exec) Unknown operation ${operation.op}`;
      }
    })();
  }

  execAnd(operation, scope) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const lhs = yield _this2.exec(operation.lhs, scope);
      const rhs = yield _this2.exec(operation.rhs, scope);
      return new _Scope2.default({
        parent: scope.parent,
        value: !!(lhs.value && rhs.value)
      });
    })();
  }

  execEq(operation, scope) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const lhs = yield _this3.exec(operation.lhs, scope);
      const rhs = yield _this3.exec(operation.rhs, scope);
      return new _Scope2.default({
        parent: scope.parent,
        value: lhs.value == rhs.value
      });
    })();
  }

  execAccessor(operation, scope) {
    return _asyncToGenerator(function* () {
      return scope.resolveAccessor(operation.path);
    })();
  }

  execLiteral(operation, scope) {
    return _asyncToGenerator(function* () {
      return new _Scope2.default({
        parent: null,
        value: operation.value
      });
    })();
  }

  execFilter(operation, scope) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      // Is this a fetch operation?
      let source;
      if (operation.source.op == 'everything') {
        const documents = yield _this4.fetch(operation.filter);
        source = new _Scope2.default({
          parent: scope,
          value: documents
        });
      } else {
        source = yield exec(operation.source, scope);
      }
      // Extract values. Might be an array or a single value.
      const items = Array.isArray(source.value) ? source.value : [source.value];

      (0, _debug2.default)("fetched:", items);

      // Run the filter on every item
      const result = [];
      for (let i = 0; i < items.length; i++) {
        const item = new _Scope2.default({
          parent: source.parent,
          value: items[i]
        });
        const filterValue = operation.filter ? yield _this4.exec(operation.filter, item) : new _Scope2.default({
          value: true
        });
        if (filterValue.value) {
          result.push(item.value);
        }
      }
      (0, _debug2.default)("filter result:", result);
      return new _Scope2.default({
        parent: scope,
        value: result
      });
    })();
  }
};
//# sourceMappingURL=Executor.js.map